// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 52088 $
// $Date: 2022-11-23 10:57:37 -0600 (Wed, 23 Nov 2022) $

// DIO24: Device Driver: source file

#include "main.h"



//*****************************************************************************
static int _query(const dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:	arg[0]	= (s32) DIO24_IOCTL_QUERY_ERROR;
					ret		= -EINVAL;
					break;

		case DIO24_QUERY_CLOCK_COUNTER:	arg[0]	= dev->cache.clock_counter;		break;
		case DIO24_QUERY_COUNT:			arg[0]	= (s32) DIO24_QUERY_LAST;		break;
		case DIO24_QUERY_DEVICE_TYPE:	arg[0]	= (s32) dev->board_type;		break;
		case DIO24_QUERY_DI:			arg[0]	= dev->cache.dedicated_input;	break;
		case DIO24_QUERY_IRQ_QTY:		arg[0]	= dev->cache.irq_qty;			break;
		case DIO24_QUERY_JUMPER_QTY:	arg[0]	= dev->cache.jumper_qty;		break;
		case DIO24_QUERY_JUMPER_SENSE:	arg[0]	= dev->cache.jumper_sense;		break;
		case DIO24_QUERY_JUMPER_VAL:	arg[0]	= dev->cache.jumper_val;		break;
	}

	return(ret);
}



//*****************************************************************************
static int _clock_latch_rise_rx(dev_data_t* dev, u32* arg)
{
	int	ret	= 0;

	if (dev->cache.irq_qty == 0)
	{
		arg[0]	= 0xFFFFFFFF;
	}
	else if ((arg[0] < 0) || (arg[0] > 3))
	{
		ret	= -EINVAL;
	}
	else
	{
		arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_cclrixr_32[arg[0]]);
	}

	return(ret);
}



//*****************************************************************************
static int _gpio_dir_out(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x3FF, dev->vaddr.gsc_iocr_32, 9, 0);
	return(ret);
}



//*****************************************************************************
static int _gpio_rx(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x1FFFFFF, dev->vaddr.gsc_ddir_32, 24, 0);

	if (dev->cache.dedicated_input == 0)
		arg[0]	&= 0xFFFFFF;

	return(ret);
}



//*****************************************************************************
static int _gpio_tx(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_ddor_32, 23, 0);
	return(ret);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	// Initialization completes within a single write cycle.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 1);
	return(0);
}



//*****************************************************************************
static int _irq_fall_enable(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.irq_qty)
	{
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_ifcr_32, 23, 0);

		if (ret == 0)
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D8, D8);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _irq_rise_enable(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.irq_qty)
	{
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_ircr_32, 23, 0);

		if (ret == 0)
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D8, D8);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _clock_count_rx(dev_data_t* dev, u32* arg)
{
	if (dev->cache.clock_counter)
	{
		arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ccr_32);
	}
	else
	{
		arg[0]	= 0xFFFFFFFF;
	}

	return(0);
}



//*****************************************************************************
static int _clock_latch_fall_rx(dev_data_t* dev, u32* arg)
{
	int	ret	= 0;

	if (dev->cache.clock_counter == 0)
	{
		arg[0]	= 0xFFFFFFFF;
	}
	else if ((arg[0] < 0) || (arg[0] > 3))
	{
		ret	= -EINVAL;
	}
	else
	{
		arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_cclfixr_32[arg[0]]);
	}

	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ DIO24_IOCTL_REG_READ,				(void*) gsc_reg_read_ioctl		},
	{ DIO24_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ DIO24_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ DIO24_IOCTL_QUERY,				(void*) _query					},
	{ DIO24_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ DIO24_IOCTL_GPIO_DIR_OUT,			(void*) _gpio_dir_out			},
	{ DIO24_IOCTL_GPIO_RX,				(void*) _gpio_rx				},
	{ DIO24_IOCTL_GPIO_TX,				(void*) _gpio_tx				},
	{ DIO24_IOCTL_IRQ_FALL_ENABLE,		(void*) _irq_fall_enable		},
	{ DIO24_IOCTL_IRQ_RISE_ENABLE,		(void*) _irq_rise_enable		},
	{ DIO24_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ DIO24_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ DIO24_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},
	{ DIO24_IOCTL_CLOCK_COUNT_RX,		(void*) _clock_count_rx			},
	{ DIO24_IOCTL_CLOCK_LATCH_FALL_RX,	(void*) _clock_latch_fall_rx	},
	{ DIO24_IOCTL_CLOCK_LATCH_RISE_RX,	(void*) _clock_latch_rise_rx	},
	{ -1, NULL }
};


