// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/driver/irq.c $
// $Rev: 44974 $
// $Date: 2019-05-22 16:45:46 -0500 (Wed, 22 May 2019) $

// DIO24: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	u32	count	= 0;
	int	disable	= 0;
	u32	bsr;
	u32	ifsr;
	u32	irsr;
	u32	is;

	for (;;)
	{
		bsr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_bsr_32);
		is	= bsr & (D9 | D10);

		if (is == 0)
		{
			if (count == 0)
			{
				// We don't know the source of the interrupt.
				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
			}

			break;
		}

		if (is & D9)
		{
			// There are one or more Rising interrupts.
			// This corresponds to the gsc_wait_t.io field.
			irsr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_irsr_32);
			irsr	&= 0xFFFFFF;

			if (irsr)
			{
				// Clear the interrupt.
				os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_irsr_32, irsr);

				// Resume any threads awiting any of these interrupts.
				gsc_wait_resume_irq_gsc(dev, irsr);
			}

			if (disable)
				os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_ircr_32, 0, irsr);
		}

		if (is & D10)
		{
			// There are one or more Falling interrupts.
			// This corresponds to the gsc_wait_t.io field.
			ifsr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_ifsr_32);
			ifsr	&= 0xFFFFFF;

			if (ifsr)
			{
				// Clear the interrupt.
				os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_ifsr_32, ifsr);

				// Resume any threads awiting any of these interrupts.
				gsc_wait_resume_irq_alt(dev, ifsr);
			}

			if (disable)
				os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_ifcr_32, 0, ifsr);
		}

		if (disable)
			break;

		count++;

		if (count >= 100)
			disable	= 1;
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ifcr_32, 0);			// Disable IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ifsr_32, 0xFFFFFF);	// Clear IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ircr_32, 0);			// Disable IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_irsr_32, 0xFFFFFF);	// Clear IRQs.
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_ifcr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ifcr_32, 0);			// Disable IRQs.

	if (dev->vaddr.gsc_ifsr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ifsr_32, 0xFFFFFF);	// Clear IRQs.

	if (dev->vaddr.gsc_ircr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ircr_32, 0);			// Disable IRQs.

	if (dev->vaddr.gsc_irsr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_irsr_32, 0xFFFFFF);	// Clear IRQs.

	gsc_irq_destroy(dev);
}


