// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/driver/main.h $
// $Rev: 52984 $
// $Date: 2023-05-03 16:40:33 -0500 (Wed, 03 May 2023) $

// DIO24: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "dio24.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_PROC_ID_STR
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"DIO24"		// Upper case form of the below.
#define	DEV_NAME							"dio24"		// MUST AGREE WITH DIO24_BASE_NAME
#define	DEV_NAME_LC							DEV_NAME
#define	DEV_NAME_UC							DEV_MODEL

#define	DEV_VERSION_STR						GSC_STR(DEV_VERSION_MAJ_NUM) "." GSC_STR(DEV_VERSION_MIN_NUM)
#define	DEV_VERSION							DEV_VERSION_STR

#define	DEV_VERSION_MAJ_NUM					9
#define	DEV_VERSION_MIN_NUM					8
// 9.8	Updated to support the 6.x series kernel.
// 9.7	Updated comments for consistency.
// 9.6	Fixed a bug in the Clock Counter feature usage logic.
//		Remove DMA calls as DMA is not used.
//		Fixed a bug in the Clock Count Rx service.
//		Updated for porting to Windows.
//		Corrected expected access writes to BAR0.
//		Updated various comments.
//		Linted sources.
//		Corrected BAR0 access permissions.
// 9.5	Added support for the 5.x kernel series.
//		Added support for Big Endian hosts.
//		Added clock latching support.
// 9.4	Added interrupt support.
//		Added wait event support.
//		Added corresponding query options.
// 9.3	Modified code so open succeeds even if initialization fails.
// 9.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Added IOCTL services.
//		Removed IRQ and WAIT support code (not supported by this device).
//		Updated user jumper calculation code.
// 9.1	Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Corrected the IOCTL code for the register write and mod services.
// 9.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		?cThe BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
// 8.1	Reduced #include list in driver interface header.
// 8.0	LINTed the source files.
//		The release version number now follows the driver version number.
// 7.0	Initial release.

// WAIT services
#define	DEV_WAIT_GSC_ALL					DIO24_WAIT_RISE_ALL
#define	DEV_WAIT_ALT_ALL					DIO24_WAIT_FALL_ALL
#define	DEV_WAIT_IO_ALL						0



// data types *****************************************************************

// This is not used, but is required.
struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).


	// Initialized by I/O service at time of use.

	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	u32					io_reg_offset;		// Offset of device's I/O FIFO.

	// Initialize by GSC commone create code.

};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.
	char				proc_id_str[8];	// User Jumper setting, if supported.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For interrupts only.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{
		// The DMA registers are not used, but required.
		VADDR_T			plx_intcsr_32;		// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;		// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;		// DMA Threshold Register

		VADDR_T			gsc_bcr_32;			// Board Control Register
		VADDR_T			gsc_bsr_32;			// Board Status Register

		VADDR_T			gsc_ccr_32;			// Clock Count Register
		VADDR_T			gsc_cclfixr_32[4];	// Clock Counter Latch Falling Interrupt x Register
		VADDR_T			gsc_cclrixr_32[4];	// Clock Counter Latch Rising Interrupt x Register

		VADDR_T			gsc_ddir_32;		// Discrete Data Input Register
		VADDR_T			gsc_ddor_32;		// Discrete Data Output Register

		VADDR_T			gsc_frr_32;			// Firmware Revision Register

		VADDR_T			gsc_ifcr_32;		// Interrupt Falling Control Register
		VADDR_T			gsc_ifsr_32;		// Interrupt Falling Status Register
		VADDR_T			gsc_iocr_32;		// I/O Control Register
		VADDR_T			gsc_ircr_32;		// Interrupt Rising Control Register
		VADDR_T			gsc_irsr_32;		// Interrupt Rising Status Register

	} vaddr;

	struct
	{
		s32				clock_counter;		// Is the Clock Counter feature supported?

		s32				dedicated_input;	// Is the Dedicated Input installed?

		u32				frr;				// Firmware Revision Register

		s32				irq_qty;			// Number of firmware supported interrupts.

		s32				jumper_qty;
		s32				jumper_sense;
		s32				jumper_val;			// FRR D16-D17
	} cache;
};



// prototypes *****************************************************************

int		dev_irq_create(dev_data_t* dev);
void	dev_irq_destroy(dev_data_t* dev);

int		initialize_ioctl(dev_data_t* dev, void* arg);



#endif
