// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/samples/irq/irq.c $
// $Rev: 49894 $
// $Date: 2022-01-12 12:00:23 -0600 (Wed, 12 Jan 2022) $

// DIO24: Sample Application: source file

#include "main.h"




// data types *****************************************************************

typedef struct
{
	const char*	name;
	s32			main;
	s32			rise;
	s32			fall;
	int			(*func)(int fd);
} irq_test_t;



// variables ******************************************************************

static	int				_fd;
static	os_sem_t		_sem_enter;
static	os_sem_t		_sem_exit;
static	int				_thread_done;
static	char			_thread_err[256];
static	dio24_wait_t	_thread_wait;

static const irq_test_t	_tests[]	=
{
	// Main interrupts
	// name						main				rise	fall	func
	{ "PCI",					GSC_WAIT_MAIN_PCI,	0,		0,		main_pci_irq_test	},
	{ "GSC",					GSC_WAIT_MAIN_GSC,	0,		0,		main_gsc_irq_test	},

	// GSC Rise Interrupts
	// name						main				rise	fall	func
	{ "Rise D0",				0,					D0,		0,		gsc_rise_d0_test	},
	{ "Rise D1",				0,					D1,		0,		gsc_rise_d1_test	},
	{ "Rise D2",				0,					D2,		0,		gsc_rise_d2_test	},
	{ "Rise D3",				0,					D3,		0,		gsc_rise_d3_test	},
	{ "Rise D4",				0,					D4,		0,		gsc_rise_d4_test	},
	{ "Rise D5",				0,					D5,		0,		gsc_rise_d5_test	},
	{ "Rise D6",				0,					D6,		0,		gsc_rise_d6_test	},
	{ "Rise D7",				0,					D7,		0,		gsc_rise_d7_test	},
	{ "Rise D8",				0,					D8,		0,		gsc_rise_d8_test	},
	{ "Rise D9",				0,					D9,		0,		gsc_rise_d9_test	},

	{ "Rise D10",				0,					D10,	0,		gsc_rise_d10_test	},
	{ "Rise D11",				0,					D11,	0,		gsc_rise_d11_test	},
	{ "Rise D12",				0,					D12,	0,		gsc_rise_d12_test	},
	{ "Rise D13",				0,					D13,	0,		gsc_rise_d13_test	},
	{ "Rise D14",				0,					D14,	0,		gsc_rise_d14_test	},
	{ "Rise D15",				0,					D15,	0,		gsc_rise_d15_test	},
	{ "Rise D16",				0,					D16,	0,		gsc_rise_d16_test	},
	{ "Rise D17",				0,					D17,	0,		gsc_rise_d17_test	},
	{ "Rise D18",				0,					D18,	0,		gsc_rise_d18_test	},
	{ "Rise D19",				0,					D19,	0,		gsc_rise_d19_test	},

	{ "Rise D20",				0,					D20,	0,		gsc_rise_d20_test	},
	{ "Rise D21",				0,					D21,	0,		gsc_rise_d21_test	},
	{ "Rise D22",				0,					D22,	0,		gsc_rise_d22_test	},
	{ "Rise D23",				0,					D23,	0,		gsc_rise_d23_test	},

	// GSC Fall Interrupts
	// name						main				rise	fall	func
	{ "Fall D0",				0,					0,		D0,		gsc_fall_d0_test	},
	{ "Fall D1",				0,					0,		D1,		gsc_fall_d1_test	},
	{ "Fall D2",				0,					0,		D2,		gsc_fall_d2_test	},
	{ "Fall D3",				0,					0,		D3,		gsc_fall_d3_test	},
	{ "Fall D4",				0,					0,		D4,		gsc_fall_d4_test	},
	{ "Fall D5",				0,					0,		D5,		gsc_fall_d5_test	},
	{ "Fall D6",				0,					0,		D6,		gsc_fall_d6_test	},
	{ "Fall D7",				0,					0,		D7,		gsc_fall_d7_test	},
	{ "Fall D8",				0,					0,		D8,		gsc_fall_d8_test	},
	{ "Fall D9",				0,					0,		D9,		gsc_fall_d9_test	},

	{ "Fall D10",				0,					0,		D10,	gsc_fall_d10_test	},
	{ "Fall D11",				0,					0,		D11,	gsc_fall_d11_test	},
	{ "Fall D12",				0,					0,		D12,	gsc_fall_d12_test	},
	{ "Fall D13",				0,					0,		D13,	gsc_fall_d13_test	},
	{ "Fall D14",				0,					0,		D14,	gsc_fall_d14_test	},
	{ "Fall D15",				0,					0,		D15,	gsc_fall_d15_test	},
	{ "Fall D16",				0,					0,		D16,	gsc_fall_d16_test	},
	{ "Fall D17",				0,					0,		D17,	gsc_fall_d17_test	},
	{ "Fall D18",				0,					0,		D18,	gsc_fall_d18_test	},
	{ "Fall D19",				0,					0,		D19,	gsc_fall_d19_test	},

	{ "Fall D20",				0,					0,		D20,	gsc_fall_d20_test	},
	{ "Fall D21",				0,					0,		D21,	gsc_fall_d21_test	},
	{ "Fall D22",				0,					0,		D22,	gsc_fall_d22_test	},
	{ "Fall D23",				0,					0,		D23,	gsc_fall_d23_test	},

	{ NULL }
};



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int			errs;
	irq_test_t*	ptr		= (void*) arg;

	os_sem_unlock(&_sem_enter);

	memset(&_thread_wait, 0, sizeof(_thread_wait));
	_thread_wait.main		= ptr->main;
	_thread_wait.rise		= ptr->rise;
	_thread_wait.fall		= ptr->fall;
	_thread_wait.timeout_ms	= 15000;
	errs					= dio24_wait_event(_fd, -1, 0, &_thread_wait);

	if (errs)
		sprintf(_thread_err, "%d. Wait request failed.", __LINE__);

	os_sem_unlock(&_sem_exit);

	_thread_done	= 1;
	return(errs);
}



//*****************************************************************************
static int _wait_for_waiting(int fd, const irq_test_t* ptr)
{
	int				errs	= 0;
	int				i;
	int				limit	= 1000;
	dio24_wait_t	wait;

	memset(&wait, 0, sizeof(wait));
	wait.main	= ptr->main;
	wait.rise	= ptr->rise;
	wait.fall	= ptr->fall;

	for (i = 0;; i++)
	{
		errs	= dio24_wait_status(fd, -1, 0, &wait);

		if (errs)
		{
			printf("FAIL <---  (%d. Wait status request failed.)\n", __LINE__);
			break;
		}
		else if (i > limit)
		{
			printf("FAIL <---  (%d. Excessive delay for thread to wait.)\n", __LINE__);
			errs++;
			break;
		}
		else if (wait.count)
		{
			break;
		}
		else
		{
			os_sleep_ms(10);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _wait_for_done(void)
{
	int	errs	= 0;
	int	i;

	for (i = 0;; i++)
	{
		if (_thread_done)
			break;

		if (i > 1000)
		{
			printf("FAIL <---  (%d. Excessive delay for thread to complete.)\n", __LINE__);
			errs++;
			break;
		}
		else
		{
			os_sleep_ms(10);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _wait_cancel(int fd)
{
	int				errs	= 0;
	int				ret;
	dio24_wait_t	wait;

	memset(&wait, 0, sizeof(wait));
	wait.main	= 0xFFFFFFFF;
	wait.rise	= 0xFFFFFFFF;
	wait.fall	= 0xFFFFFFFF;
	ret			= dio24_wait_cancel(fd, -1, 0, &wait);
	errs		+= ret ? 1 : 0;

	return(errs);
}



//*****************************************************************************
static int _test_irq(int fd, const irq_test_t* ptr)
{
	int			errs		= 0;
	int			ret;
	os_thread_t	thread;

	for (;;)	// A cenvenience loop.
	{
		gsc_label(ptr->name);

		// Prepare for testing the interrupt.
		_thread_err[0]	= 0;
		_thread_done	= 0;
		_fd				= fd;
		memset(&_thread_wait, 0, sizeof(_thread_wait));
		os_sem_create_qty(&_sem_enter, 1, 0);
		os_sem_create_qty(&_sem_exit, 1, 0);

		// Create a thread to wait on the interrupt.
		errs	= os_thread_create(&thread, "wait thtread", _wait_thread, (void*) ptr);

		if (errs)
		{
			printf("FAIL <---  (%d. Thread creation failed.)\n", __LINE__);
			break;
		}

		// Wait for the thread to start running.
		ret	= os_sem_lock(&_sem_enter);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%d. Semaphore 'enter' lock failure.)\n", __LINE__);
			_wait_cancel(fd);
			os_sem_lock(&_sem_exit);
			_wait_for_done();
			os_thread_destroy(&thread);
			break;
		}

		// Wait for the thread to become "waiting".
		errs	= _wait_for_waiting(fd, ptr);

		if (errs)
		{
			_wait_cancel(fd);
			os_sem_lock(&_sem_exit);
			_wait_for_done();
			os_thread_destroy(&thread);
			break;
		}

		// Initiate the interrupt.
		errs			= ptr->func(fd);

		if (errs)
		{
			printf("FAIL <---  (%d. Interrupt function failure.)\n", __LINE__);
			_wait_cancel(fd);
			os_sem_lock(&_sem_exit);
			_wait_for_done();
			os_thread_destroy(&thread);
			break;
		}

		// Wait for the thread to stop running.
		ret	= os_sem_lock(&_sem_exit);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%d. Semaphore 'exit' lock failure.)\n", __LINE__);
			_wait_cancel(fd);
			_wait_for_done();
			os_thread_destroy(&thread);
			break;
		}

		// End processing.
		_wait_cancel(fd);	// just in case
		_wait_for_done();
		os_thread_destroy(&thread);

		if (_thread_err[0])
		{
			errs++;
			printf("FAIL <---  (%d. Failure in wait thread: %s)\n", __LINE__, _thread_err);
			break;
		}

		if (_thread_wait.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			errs++;
			printf("FAIL <---  (%d. Wait for interrupt timed out.)\n", __LINE__);
			break;
		}

		if (_thread_wait.flags & GSC_WAIT_FLAG_CANCEL)
		{
			errs++;
			printf("FAIL <---  (%d. Wait for interrupt was cancelled.)\n", __LINE__);
			break;
		}

		if (_thread_wait.flags & ~GSC_WAIT_FLAG_DONE)
		{
			errs++;
			printf("FAIL <---  (%d. Invalid wait flags results: 0x%lX)\n", __LINE__, (long) _thread_wait.flags);
			break;
		}

		if (_thread_wait.flags & GSC_WAIT_FLAG_DONE)
		{
			printf("PASS\n");
			break;
		}

		errs++;
		printf("FAIL <---  (%d. Invalid wait flags results: 0x%lX)\n", __LINE__, (long) _thread_wait.flags);
		break;
	}

	os_sem_destroy(&_sem_enter);
	os_sem_destroy(&_sem_exit);
	return(errs);
}



//*****************************************************************************
int irq_tests(int fd)
{
	int	errs;
	int	i;
	s32	irq;

	errs	= dio24_query(fd, -1, 0, DIO24_QUERY_IRQ_QTY, &irq);
	gsc_label("IRQ Tests");

	if (errs)
	{
		printf("ABORTING DUE TO ERROR.\n");
	}
	else if (irq <= 0)
	{
		printf("SKIPING  (Interrupts not supported by firmware.)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		for (i = 0; _tests[i].name; i++)
			errs	+= _test_irq(fd, &_tests[i]);

		gsc_label_level_dec();
	}

	return(errs);
}


